=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2011-09-15 15:07:02 +0000
+++ bzrlib/__init__.py	2011-09-26 15:40:02 +0000
@@ -137,7 +137,7 @@
 # variables that need to be hunted down. Accessing the current BzrLibraryState
 # through this variable is not encouraged: it is better to pass it around as
 # part of the context of an operation than to look it up directly, but when
-# that is too hard, it is better to use this variable than to make a branch new
+# that is too hard, it is better to use this variable than to make a brand new
 # global variable.
 # If using this variable by looking it up (because it can't be easily obtained)
 # it is important to store the reference you get, rather than looking it up

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2011-09-27 11:48:50 +0000
+++ bzrlib/commands.py	2011-09-29 15:50:58 +0000
@@ -663,7 +663,6 @@
             opts['quiet'] = trace.is_quiet()
         elif opts.has_key('quiet'):
             del opts['quiet']
-
         # mix arguments and options into one dictionary
         cmdargs = _match_argform(self.name(), self.takes_args, args)
         cmdopts = {}
@@ -1041,6 +1040,7 @@
 
     argv_copy = []
     i = 0
+    override_config = []
     while i < len(argv):
         a = argv[i]
         if a == '--profile':
@@ -1069,10 +1069,14 @@
             pass # already handled in startup script Bug #588277
         elif a.startswith('-D'):
             debug.debug_flags.add(a[2:])
+        elif a.startswith('-O'):
+            override_config.append(a[2:])
         else:
             argv_copy.append(a)
         i += 1
 
+    bzrlib.global_state.cmdline_overrides._from_cmdline(override_config)
+
     debug.set_debug_flags_from_config()
 
     if not opt_no_plugins:
@@ -1130,6 +1134,8 @@
         if 'memory' in debug.debug_flags:
             trace.debug_memory('Process status after command:', short=False)
         option._verbosity_level = saved_verbosity_level
+        # Reset the overrides 
+        bzrlib.global_state.cmdline_overrides._reset()
 
 
 def display_command(func):
@@ -1164,8 +1170,9 @@
         "bzr plugin commands")
     Command.hooks.install_named_hook("get_command", _get_external_command,
         "bzr external command lookup")
-    Command.hooks.install_named_hook("get_missing_command", _try_plugin_provider,
-        "bzr plugin-provider-db check")
+    Command.hooks.install_named_hook("get_missing_command",
+                                     _try_plugin_provider,
+                                     "bzr plugin-provider-db check")
 
 
 

=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2011-09-26 07:56:05 +0000
+++ bzrlib/config.py	2011-09-29 15:50:58 +0000
@@ -77,6 +77,7 @@
 import sys
 
 
+import bzrlib
 from bzrlib.decorators import needs_write_lock
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
@@ -101,6 +102,7 @@
     urlutils,
     win32utils,
     )
+from bzrlib.i18n import gettext
 from bzrlib.util.configobj import configobj
 """)
 from bzrlib import (
@@ -2654,6 +2656,31 @@
         del self.options[name]
 
 
+class CommandLineSection(MutableSection):
+    """A section used to carry command line overrides for the config options."""
+
+    def __init__(self, opts=None):
+        if opts is None:
+            opts = {}
+        super(CommandLineSection, self).__init__('cmdline-overrides', opts)
+
+    def _reset(self):
+        # The dict should be cleared but not replaced so it can be shared.
+        self.options.clear()
+
+    def _from_cmdline(self, overrides):
+        # Reset before accepting new definitions
+        self._reset()
+        for over in overrides:
+            try:
+                name, value = over.split('=', 1)
+            except ValueError:
+                raise errors.BzrCommandError(
+                    gettext("Invalid '%s', should be of the form 'name=value'")
+                    % (over,))
+            self.set(name, value)
+
+
 class Store(object):
     """Abstract interface to persistent storage for configuration options."""
 
@@ -3275,7 +3302,9 @@
     def __init__(self):
         # Get a GlobalStore
         gstore = GlobalStore()
-        super(GlobalStack, self).__init__([gstore.get_sections], gstore)
+        super(GlobalStack, self).__init__(
+            [bzrlib.global_state.cmdline_overrides, gstore.get_sections],
+            gstore)
 
 
 class LocationStack(_CompatibleStack):
@@ -3289,7 +3318,9 @@
         matcher = LocationMatcher(lstore, location)
         gstore = GlobalStore()
         super(LocationStack, self).__init__(
-            [matcher.get_sections, gstore.get_sections], lstore)
+            [bzrlib.global_state.cmdline_overrides,
+             matcher.get_sections, gstore.get_sections],
+            lstore)
 
 
 class BranchStack(_CompatibleStack):
@@ -3301,7 +3332,8 @@
         matcher = LocationMatcher(lstore, branch.base)
         gstore = GlobalStore()
         super(BranchStack, self).__init__(
-            [matcher.get_sections, bstore.get_sections, gstore.get_sections],
+            [bzrlib.global_state.cmdline_overrides,
+             matcher.get_sections, bstore.get_sections, gstore.get_sections],
             bstore)
         self.branch = branch
 

=== modified file 'bzrlib/help_topics/__init__.py'
--- bzrlib/help_topics/__init__.py	2011-09-07 16:32:11 +0000
+++ bzrlib/help_topics/__init__.py	2011-09-29 13:54:10 +0000
@@ -331,6 +331,10 @@
                will be a pickle.
 --coverage     Generate line coverage report in the specified directory.
 
+-Oname=value   Override the ``name`` config option setting it to ``value`` for
+               the duration of the command.  This can be used multiple times if
+               several options need to be overridden.
+
 See http://doc.bazaar.canonical.com/developers/profiling.html for more
 information on profiling.
 

=== modified file 'bzrlib/library_state.py'
--- bzrlib/library_state.py	2011-03-29 09:50:04 +0000
+++ bzrlib/library_state.py	2011-09-26 15:40:02 +0000
@@ -23,6 +23,10 @@
 import sys
 
 import bzrlib
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+from bzrlib import config
+""")
 
 
 class BzrLibraryState(object):
@@ -61,6 +65,9 @@
         """
         self._ui = ui
         self._trace = trace
+        # There is no overrides by default, they are set later when the command
+        # arguments are parsed.
+        self.cmdline_overrides = config.CommandLineSection()
         self.started = False
 
     def __enter__(self):
@@ -69,8 +76,7 @@
         return self # This is bound to the 'as' clause in a with statement.
 
     def _start(self):
-        """Do all initialization.
-        """        
+        """Do all initialization."""
         # NB: This function tweaks so much global state it's hard to test it in
         # isolation within the same interpreter.  It's not reached on normal
         # in-process run_bzr calls.  If it's broken, we expect that

=== modified file 'bzrlib/option.py'
--- bzrlib/option.py	2011-09-22 14:30:17 +0000
+++ bzrlib/option.py	2011-09-27 13:18:53 +0000
@@ -475,6 +475,12 @@
     Option.STD_OPTIONS[name] = Option(name, **kwargs)
     Option.OPTIONS[name] = Option.STD_OPTIONS[name]
 
+def _standard_list_option(name, **kwargs):
+    """Register a standard option."""
+    # All standard options are implicitly 'global' ones
+    Option.STD_OPTIONS[name] = ListOption(name, **kwargs)
+    Option.OPTIONS[name] = Option.STD_OPTIONS[name]
+
 
 def _global_option(name, **kwargs):
     """Register a global option."""
@@ -531,14 +537,14 @@
 # Declare the standard options
 _standard_option('help', short_name='h',
                  help='Show help message.')
+_standard_option('quiet', short_name='q',
+                 help="Only display errors and warnings.",
+                 custom_callback=_verbosity_level_callback)
 _standard_option('usage',
                  help='Show usage message and options.')
 _standard_option('verbose', short_name='v',
                  help='Display more information.',
                  custom_callback=_verbosity_level_callback)
-_standard_option('quiet', short_name='q',
-                 help="Only display errors and warnings.",
-                 custom_callback=_verbosity_level_callback)
 
 # Declare commonly used options
 _global_option('all')

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2011-09-26 07:56:05 +0000
+++ bzrlib/tests/__init__.py	2011-09-29 15:50:58 +0000
@@ -983,6 +983,8 @@
         for feature in getattr(self, '_test_needs_features', []):
             self.requireFeature(feature)
         self._cleanEnvironment()
+        self.overrideAttr(bzrlib.global_state, 'cmdline_overrides',
+                          config.CommandLineSection())
         self._silenceUI()
         self._startLogFile()
         self._benchcalls = []

=== modified file 'bzrlib/tests/test_commands.py'
--- bzrlib/tests/test_commands.py	2011-07-15 14:27:20 +0000
+++ bzrlib/tests/test_commands.py	2011-09-29 13:54:10 +0000
@@ -91,6 +91,23 @@
         self.assertContainsRe(c.get_help_text(), '--foo')
 
 
+class TestInsideCommand(tests.TestCaseInTempDir):
+
+    def test_command_see_config_overrides(self):
+        def run(cmd):
+            # We override the run() command method so we can observe the
+            # overrides from inside.
+            c = config.GlobalStack()
+            self.assertEquals('12', c.get('xx'))
+            self.assertEquals('foo', c.get('yy'))
+        self.overrideAttr(builtins.cmd_rocks, 'run', run)
+        self.run_bzr(['rocks', '-Oxx=12', '-Oyy=foo'])
+        c = config.GlobalStack()
+        # Ensure that we don't leak outside of the command
+        self.assertEquals(None, c.get('xx'))
+        self.assertEquals(None, c.get('yy'))
+
+
 class TestInvokedAs(tests.TestCase):
 
     def test_invoked_as(self):

=== modified file 'bzrlib/tests/test_config.py'
--- bzrlib/tests/test_config.py	2011-09-13 12:29:22 +0000
+++ bzrlib/tests/test_config.py	2011-09-26 15:40:02 +0000
@@ -2562,12 +2562,17 @@
 
 class TestMutableSection(tests.TestCase):
 
-    # FIXME: Parametrize so that all sections (including os.environ and the
-    # ones produced by Stores) run these tests -- vila 2011-04-01
+    scenarios = [('mutable',
+                  {'get_section':
+                       lambda opts: config.MutableSection('myID', opts)},),
+                 ('cmdline',
+                  {'get_section':
+                       lambda opts: config.CommandLineSection(opts)},),
+        ]
 
     def test_set(self):
         a_dict = dict(foo='bar')
-        section = config.MutableSection('myID', a_dict)
+        section = self.get_section(a_dict)
         section.set('foo', 'new_value')
         self.assertEquals('new_value', section.get('foo'))
         # The change appears in the shared section
@@ -2578,7 +2583,7 @@
 
     def test_set_preserve_original_once(self):
         a_dict = dict(foo='bar')
-        section = config.MutableSection('myID', a_dict)
+        section = self.get_section(a_dict)
         section.set('foo', 'first_value')
         section.set('foo', 'second_value')
         # We keep track of the original value
@@ -2587,7 +2592,7 @@
 
     def test_remove(self):
         a_dict = dict(foo='bar')
-        section = config.MutableSection('myID', a_dict)
+        section = self.get_section(a_dict)
         section.remove('foo')
         # We get None for unknown options via the default value
         self.assertEquals(None, section.get('foo'))
@@ -2600,7 +2605,7 @@
 
     def test_remove_new_option(self):
         a_dict = dict()
-        section = config.MutableSection('myID', a_dict)
+        section = self.get_section(a_dict)
         section.set('foo', 'bar')
         section.remove('foo')
         self.assertFalse('foo' in section.options)
@@ -2610,6 +2615,40 @@
         self.assertEquals(config._NewlyCreatedOption, section.orig['foo'])
 
 
+class TestCommandLineSection(tests.TestCase):
+
+    def setUp(self):
+        super(TestCommandLineSection, self).setUp()
+        self.section = config.CommandLineSection()
+
+    def test_no_override(self):
+        self.section._from_cmdline([])
+        # FIXME: we want some iterator over all options, failing that, we peek
+        # under the cover -- vila 2011-09026
+        self.assertLength(0, self.section.options)
+
+    def test_simple_override(self):
+        self.section._from_cmdline(['a=b'])
+        self.assertEqual('b', self.section.get('a'))
+
+    def test_list_override(self):
+        self.section._from_cmdline(['l=1,2,3'])
+        val = self.section.get('l')
+        self.assertEqual('1,2,3', val)
+        # Reminder: lists should registered as such explicitely, otherwise the
+        # conversion needs to be done afterwards.
+        self.assertEqual(['1', '2', '3'], config.list_from_store(val))
+
+    def test_multiple_overrides(self):
+        self.section._from_cmdline(['a=b', 'x=y'])
+        self.assertEquals('b', self.section.get('a'))
+        self.assertEquals('y', self.section.get('x'))
+
+    def test_wrong_syntax(self):
+        self.assertRaises(errors.BzrCommandError,
+                          self.section._from_cmdline, ['a=b', 'c'])
+
+
 class TestStore(tests.TestCaseWithTransport):
 
     def assertSectionContent(self, expected, section):

=== modified file 'bzrlib/tests/test_help.py'
--- bzrlib/tests/test_help.py	2011-09-02 16:34:17 +0000
+++ bzrlib/tests/test_help.py	2011-09-27 13:18:53 +0000
@@ -45,17 +45,17 @@
             __doc__ = """A sample command."""
             _see_also = ['foo', 'bar']
         self.assertCmdHelp('''\
-            Purpose: A sample command.
-            Usage:   bzr WithSeeAlso
-            
-            Options:
-              --usage        Show usage message and options.
-              -v, --verbose  Display more information.
-              -q, --quiet    Only display errors and warnings.
-              -h, --help     Show help message.
-            
-            See also: bar, foo
-            ''',
+Purpose: A sample command.
+Usage:   bzr WithSeeAlso
+
+Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+See also: bar, foo
+''',
                            cmd_WithSeeAlso())
 
     def test_get_help_text(self):
@@ -63,16 +63,16 @@
         class cmd_Demo(commands.Command):
             __doc__ = """A sample command."""
         self.assertCmdHelp('''\
-            Purpose: A sample command.
-            Usage:   bzr Demo
-            
-            Options:
-              --usage        Show usage message and options.
-              -v, --verbose  Display more information.
-              -q, --quiet    Only display errors and warnings.
-              -h, --help     Show help message.
-
-            ''',
+Purpose: A sample command.
+Usage:   bzr Demo
+
+Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+''',
                            cmd_Demo())
         cmd = cmd_Demo()
         helptext = cmd.get_help_text()
@@ -90,8 +90,8 @@
         helptext = cmd.get_help_text(['gam'])
         self.assertEndsWith(
             helptext,
+            '  -q, --quiet    Only display errors and warnings.\n'
             '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
             '  -h, --help     Show help message.\n'
             '\n'
             'See also: bar, foo, gam\n')
@@ -103,8 +103,8 @@
         helptext = cmd.get_help_text(['gam'])
         self.assertEndsWith(
             helptext,
+            '  -q, --quiet    Only display errors and warnings.\n'
             '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
             '  -h, --help     Show help message.\n'
             '\n'
             'See also: gam\n')
@@ -138,56 +138,59 @@
             """
         cmd = cmd_Demo()
         helptext = cmd.get_help_text()
-        self.assertEquals(
-            helptext,
-            'Purpose: A sample command.\n'
-            'Usage:   bzr Demo\n'
-            '\n'
-            'Options:\n'
-            '  --usage        Show usage message and options.\n'
-            '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
-            '  -h, --help     Show help message.\n'
-            '\n'
-            'Examples:\n'
-            '    Example 1:\n'
-            '\n'
-            '        cmd arg1\n'
-            '\n'
-            '    Example 2:\n'
-            '\n'
-            '        cmd arg2\n'
-            '\n'
-            '    A code block follows.\n'
-            '\n'
-            '        bzr Demo something\n'
-            '\n')
+        self.assertEqualDiff('''\
+Purpose: A sample command.
+Usage:   bzr Demo
+
+Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+Examples:
+    Example 1:
+
+        cmd arg1
+
+    Example 2:
+
+        cmd arg2
+
+    A code block follows.
+
+        bzr Demo something
+
+''',
+                                         helptext)
         helptext = cmd.get_help_text(plain=False)
-        self.assertEquals(helptext,
-            ':Purpose: A sample command.\n'
-            ':Usage:   bzr Demo\n'
-            '\n'
-            ':Options:\n'
-            '  --usage        Show usage message and options.\n'
-            '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
-            '  -h, --help     Show help message.\n'
-            '\n'
-            ':Examples:\n'
-            '    Example 1::\n'
-            '\n'
-            '        cmd arg1\n'
-            '\n'
-            '    Example 2::\n'
-            '\n'
-            '        cmd arg2\n'
-            '\n'
-            '    A code block follows.\n'
-            '\n'
-            '    ::\n'
-            '\n'
-            '        bzr Demo something\n'
-            '\n')
+        self.assertEqualDiff('''\
+:Purpose: A sample command.
+:Usage:   bzr Demo
+
+:Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+:Examples:
+    Example 1::
+
+        cmd arg1
+
+    Example 2::
+
+        cmd arg2
+
+    A code block follows.
+
+    ::
+
+        bzr Demo something
+
+''',
+                             helptext)
 
     def test_concise_help_text(self):
         """Concise help text excludes the descriptive sections."""
@@ -203,84 +206,89 @@
             """
         cmd = cmd_Demo()
         helptext = cmd.get_help_text()
-        self.assertEqualDiff(
-            helptext,
-            'Purpose: A sample command.\n'
-            'Usage:   bzr Demo\n'
-            '\n'
-            'Options:\n'
-            '  --usage        Show usage message and options.\n'
-            '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
-            '  -h, --help     Show help message.\n'
-            '\n'
-            'Description:\n'
-            '  Blah blah blah.\n'
-            '\n'
-            'Examples:\n'
-            '    Example 1:\n'
-            '\n'
-            '        cmd arg1\n'
-            '\n')
+        self.assertEqualDiff('''\
+Purpose: A sample command.
+Usage:   bzr Demo
+
+Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+Description:
+  Blah blah blah.
+
+Examples:
+    Example 1:
+
+        cmd arg1
+
+''',
+                             helptext)
         helptext = cmd.get_help_text(verbose=False)
-        self.assertEquals(helptext,
-            'Purpose: A sample command.\n'
-            'Usage:   bzr Demo\n'
-            '\n'
-            'Options:\n'
-            '  --usage        Show usage message and options.\n'
-            '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
-            '  -h, --help     Show help message.\n'
-            '\n'
-            'See bzr help Demo for more details and examples.\n'
-            '\n')
+        self.assertEqualDiff('''\
+Purpose: A sample command.
+Usage:   bzr Demo
+
+Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+See bzr help Demo for more details and examples.
+
+''',
+                             helptext)
 
     def test_help_custom_section_ordering(self):
         """Custom descriptive sections should remain in the order given."""
         class cmd_Demo(commands.Command):
-            __doc__ = """A sample command.
- 
-            Blah blah blah.
-
-            :Formats:
-              Interesting stuff about formats.
-
-            :Examples:
-              Example 1::
- 
-                cmd arg1
-
-            :Tips:
-              Clever things to keep in mind.
-            """
+            __doc__ = """\
+A sample command.
+
+Blah blah blah.
+
+:Formats:
+  Interesting stuff about formats.
+
+:Examples:
+  Example 1::
+
+    cmd arg1
+
+:Tips:
+  Clever things to keep in mind.
+"""
         cmd = cmd_Demo()
         helptext = cmd.get_help_text()
-        self.assertEqualDiff(
-            helptext,
-            'Purpose: A sample command.\n'
-            'Usage:   bzr Demo\n'
-            '\n'
-            'Options:\n'
-            '  --usage        Show usage message and options.\n'
-            '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
-            '  -h, --help     Show help message.\n'
-            '\n'
-            'Description:\n'
-            '  Blah blah blah.\n'
-            '\n'
-            'Formats:\n'
-            '  Interesting stuff about formats.\n'
-            '\n'
-            'Examples:\n'
-            '  Example 1:\n'
-            '\n'
-            '    cmd arg1\n'
-            '\n'
-            'Tips:\n'
-            '  Clever things to keep in mind.\n'
-            '\n')
+        self.assertEqualDiff('''\
+Purpose: A sample command.
+Usage:   bzr Demo
+
+Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+Description:
+  Blah blah blah.
+
+Formats:
+  Interesting stuff about formats.
+
+Examples:
+  Example 1:
+
+    cmd arg1
+
+Tips:
+  Clever things to keep in mind.
+
+''',
+                             helptext)
 
     def test_help_text_custom_usage(self):
         """Help text may contain a custom usage section."""
@@ -296,22 +304,25 @@
             """
         cmd = cmd_Demo()
         helptext = cmd.get_help_text()
-        self.assertEquals(helptext,
-            'Purpose: A sample command.\n'
-            'Usage:\n'
-            '    cmd Demo [opts] args\n'
-            '\n'
-            '    cmd Demo -h\n'
-            '\n'
-            '\n'
-            'Options:\n'
-            '  --usage        Show usage message and options.\n'
-            '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
-            '  -h, --help     Show help message.\n'
-            '\n'
-            'Description:\n'
-            '  Blah blah blah.\n\n')
+        self.assertEqualDiff('''\
+Purpose: A sample command.
+Usage:
+    cmd Demo [opts] args
+
+    cmd Demo -h
+
+
+Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+Description:
+  Blah blah blah.
+
+''',
+                             helptext)
 
 
 class ZzzTranslationsForDoc(ZzzTranslations):
@@ -343,17 +354,17 @@
             __doc__ = """A sample command."""
             _see_also = ['foo', 'bar']
         self.assertCmdHelp('''\
-            zz{{:Purpose: zz{{A sample command.}}
-            }}zz{{:Usage:   bzr WithSeeAlso
-            }}
-            zz{{:Options:
-              --usage        zz{{Show usage message and options.}}
-              -v, --verbose  zz{{Display more information.}}
-              -q, --quiet    zz{{Only display errors and warnings.}}
-              -h, --help     zz{{Show help message.}}
-            }}
-            zz{{:See also: bar, foo}}
-            ''',
+zz{{:Purpose: zz{{A sample command.}}
+}}zz{{:Usage:   bzr WithSeeAlso
+}}
+zz{{:Options:
+  --usage        zz{{Show usage message and options.}}
+  -q, --quiet    zz{{Only display errors and warnings.}}
+  -v, --verbose  zz{{Display more information.}}
+  -h, --help     zz{{Show help message.}}
+}}
+zz{{:See also: bar, foo}}
+''',
                            cmd_WithSeeAlso())
 
     def test_get_help_text(self):
@@ -361,16 +372,16 @@
         class cmd_Demo(commands.Command):
             __doc__ = """A sample command."""
         self.assertCmdHelp('''\
-            zz{{:Purpose: zz{{A sample command.}}
-            }}zz{{:Usage:   bzr Demo
-            }}
-            zz{{:Options:
-              --usage        zz{{Show usage message and options.}}
-              -v, --verbose  zz{{Display more information.}}
-              -q, --quiet    zz{{Only display errors and warnings.}}
-              -h, --help     zz{{Show help message.}}
-            }}
-            ''',
+zz{{:Purpose: zz{{A sample command.}}
+}}zz{{:Usage:   bzr Demo
+}}
+zz{{:Options:
+  --usage        zz{{Show usage message and options.}}
+  -q, --quiet    zz{{Only display errors and warnings.}}
+  -v, --verbose  zz{{Display more information.}}
+  -h, --help     zz{{Show help message.}}
+}}
+''',
                            cmd_Demo())
 
     def test_command_with_additional_see_also(self):
@@ -380,12 +391,13 @@
         cmd = cmd_WithSeeAlso()
         helptext = cmd.get_help_text(['gam'])
         self.assertEndsWith(
-            helptext,
-            '  -v, --verbose  zz{{Display more information.}}\n'
-            '  -q, --quiet    zz{{Only display errors and warnings.}}\n'
-            '  -h, --help     zz{{Show help message.}}\n'
-            '}}\n'
-            'zz{{:See also: bar, foo, gam}}\n')
+            helptext,'''\
+  -q, --quiet    zz{{Only display errors and warnings.}}
+  -v, --verbose  zz{{Display more information.}}
+  -h, --help     zz{{Show help message.}}
+}}
+zz{{:See also: bar, foo, gam}}
+''')
 
     def test_command_only_additional_see_also(self):
         class cmd_WithSeeAlso(commands.Command):
@@ -393,14 +405,15 @@
         cmd = cmd_WithSeeAlso()
         helptext = cmd.get_help_text(['gam'])
         self.assertEndsWith(
-            helptext,
-            'zz{{:Options:\n'
-            '  --usage        zz{{Show usage message and options.}}\n'
-            '  -v, --verbose  zz{{Display more information.}}\n'
-            '  -q, --quiet    zz{{Only display errors and warnings.}}\n'
-            '  -h, --help     zz{{Show help message.}}\n'
-            '}}\n'
-            'zz{{:See also: gam}}\n')
+            helptext, '''\
+zz{{:Options:
+  --usage        zz{{Show usage message and options.}}
+  -q, --quiet    zz{{Only display errors and warnings.}}
+  -v, --verbose  zz{{Display more information.}}
+  -h, --help     zz{{Show help message.}}
+}}
+zz{{:See also: gam}}
+''')
 
 
     def test_help_custom_section_ordering(self):
@@ -423,30 +436,30 @@
               Clever things to keep in mind.
             """
         self.assertCmdHelp('''\
-            zz{{:Purpose: zz{{A sample command.}}
-            }}zz{{:Usage:   bzr Demo
-            }}
-            zz{{:Options:
-              --usage        zz{{Show usage message and options.}}
-              -v, --verbose  zz{{Display more information.}}
-              -q, --quiet    zz{{Only display errors and warnings.}}
-              -h, --help     zz{{Show help message.}}
-            }}
-            Description:
-              zz{{zz{{Blah blah blah.}}
-            
-            }}:Formats:
-              zz{{Interesting stuff about formats.}}
-            
-            Examples:
-              zz{{Example 1::}}
-            
-                zz{{cmd arg1}}
-            
-            Tips:
-              zz{{Clever things to keep in mind.}}
-
-            ''',
+zz{{:Purpose: zz{{A sample command.}}
+}}zz{{:Usage:   bzr Demo
+}}
+zz{{:Options:
+  --usage        zz{{Show usage message and options.}}
+  -q, --quiet    zz{{Only display errors and warnings.}}
+  -v, --verbose  zz{{Display more information.}}
+  -h, --help     zz{{Show help message.}}
+}}
+Description:
+  zz{{zz{{Blah blah blah.}}
+ 
+}}:Formats:
+  zz{{Interesting stuff about formats.}}
+ 
+Examples:
+  zz{{Example 1::}}
+ 
+    zz{{cmd arg1}}
+ 
+Tips:
+  zz{{Clever things to keep in mind.}}
+ 
+''',
                            cmd_Demo())
 
     def test_help_text_custom_usage(self):
@@ -462,24 +475,24 @@
             Blah blah blah.
             """
         self.assertCmdHelp('''\
-            zz{{:Purpose: zz{{A sample command.}}
-            }}zz{{:Usage:
-                zz{{cmd Demo [opts] args}}
-            
-                zz{{cmd Demo -h}}
-
-            }}
-            zz{{:Options:
-              --usage        zz{{Show usage message and options.}}
-              -v, --verbose  zz{{Display more information.}}
-              -q, --quiet    zz{{Only display errors and warnings.}}
-              -h, --help     zz{{Show help message.}}
-            }}
-            Description:
-              zz{{zz{{Blah blah blah.}}
-
-            }}
-            ''',
+zz{{:Purpose: zz{{A sample command.}}
+}}zz{{:Usage:
+    zz{{cmd Demo [opts] args}}
+
+    zz{{cmd Demo -h}}
+
+}}
+zz{{:Options:
+  --usage        zz{{Show usage message and options.}}
+  -q, --quiet    zz{{Only display errors and warnings.}}
+  -v, --verbose  zz{{Display more information.}}
+  -h, --help     zz{{Show help message.}}
+}}
+Description:
+  zz{{zz{{Blah blah blah.}}
+
+}}
+''',
                            cmd_Demo())
 
 

=== modified file 'bzrlib/tests/test_options.py'
--- bzrlib/tests/test_options.py	2011-09-27 11:48:50 +0000
+++ bzrlib/tests/test_options.py	2011-09-29 15:50:58 +0000
@@ -43,15 +43,18 @@
         # XXX: Using cmd_commit makes these tests overly sensitive to changes
         # to cmd_commit, when they are meant to be about option parsing in
         # general.
-        self.assertEqual(parse_args(cmd_commit(), ['--help']),
-           ([], {'author': [], 'exclude': [], 'fixes': [], 'help': True}))
-        self.assertEqual(parse_args(cmd_commit(), ['--message=biter']),
-           ([], {'author': [], 'exclude': [], 'fixes': [], 'message': 'biter'}))
+        self.assertEqual(
+           ([], {'author': [], 'exclude': [], 'fixes': [], 'help': True}),
+           parse_args(cmd_commit(), ['--help']))
+        self.assertEqual(
+           ([], {'author': [], 'exclude': [], 'fixes': [], 'message': 'biter'}),
+           parse_args(cmd_commit(), ['--message=biter']))
 
     def test_no_more_opts(self):
         """Terminated options"""
-        self.assertEqual(parse_args(cmd_commit(), ['--', '-file-with-dashes']),
-                          (['-file-with-dashes'], {'author': [], 'exclude': [], 'fixes': []}))
+        self.assertEqual(
+            (['-file-with-dashes'], {'author': [], 'exclude': [], 'fixes': []}),
+            parse_args(cmd_commit(), ['--', '-file-with-dashes']))
 
     def test_option_help(self):
         """Options have help strings."""

=== modified file 'doc/en/release-notes/bzr-2.5.txt'
--- doc/en/release-notes/bzr-2.5.txt	2011-09-28 15:32:55 +0000
+++ doc/en/release-notes/bzr-2.5.txt	2011-09-29 15:50:58 +0000
@@ -20,6 +20,12 @@
 
 .. New commands, options, etc that users may wish to try out.
 
+* A new ``-O`` standard option (common to all commands) have been added. It
+  provides a value for a config option in the ``-Oname=value`` form that
+  takes precedence over all definitions found in config files.  It can be
+  used multiple times to override different options.
+  (Vincent Ladeuil, #491196)
+
 * ``bzr serve`` will now disconnect clients if they have not issued an RPC
   request after 5minutes. On POSIX platforms, this will also happen for
   ``bzr serve --inet``. This can be overridden with the configuration

=== modified file 'tools/generate_docs.py'
--- tools/generate_docs.py	2009-07-14 15:01:19 +0000
+++ tools/generate_docs.py	2011-09-26 15:40:02 +0000
@@ -38,7 +38,12 @@
 
 sys.path.insert(0, os.path.dirname(os.path.dirname(__file__)))
 
-from bzrlib import commands, doc_generate
+import bzrlib
+from bzrlib import (
+    commands,
+    doc_generate,
+    )
+
 
 def main(argv):
     parser = OptionParser(usage="""%prog [options] OUTPUT_FORMAT
@@ -71,26 +76,23 @@
         parser.print_help()
         sys.exit(1)
 
-    commands.install_bzr_command_hooks()
-
-    infogen_type = args[1]
-    infogen_mod = doc_generate.get_module(infogen_type)
-
-    if options.filename:
-        outfilename = options.filename
-    else:
-        outfilename = infogen_mod.get_filename(options)
-
-    if outfilename == "-":
-        outfile = sys.stdout
-    else:
-        outfile = open(outfilename,"w")
-
-    if options.show_filename and (outfilename != "-"):
-        sys.stdout.write(outfilename)
-        sys.stdout.write('\n')
-    
-    infogen_mod.infogen(options, outfile)
+    with bzrlib.initialize():
+        commands.install_bzr_command_hooks()
+        infogen_type = args[1]
+        infogen_mod = doc_generate.get_module(infogen_type)
+        if options.filename:
+            outfilename = options.filename
+        else:
+            outfilename = infogen_mod.get_filename(options)
+        if outfilename == "-":
+            outfile = sys.stdout
+        else:
+            outfile = open(outfilename,"w")
+        if options.show_filename and (outfilename != "-"):
+            sys.stdout.write(outfilename)
+            sys.stdout.write('\n')
+        infogen_mod.infogen(options, outfile)
+
 
 def print_extended_help(option, opt, value, parser):
     """ Program help examples

